package com.afp.ordermanagement.config;

import com.afp.ordermanagement.model.Order;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.support.serializer.JsonSerializer;
import reactor.kafka.receiver.KafkaReceiver;
import reactor.kafka.receiver.ReceiverOptions;
import reactor.kafka.sender.KafkaSender;
import reactor.kafka.sender.SenderOptions;



import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

@Configuration
public class KafkaConfig {

    @Value("${kafka.producer.bootstrap-servers}")
    private String bootstrapServers;

    @Value("${kafka.producer.acks}")
    private String acks;

    @Value("${kafka.consumer.group-id}")
    private String groupId;


    @Bean
    public Map<String, Object> producerFactoryString() {
        Map<String, Object> senderConfigProps = new HashMap<>();
        senderConfigProps.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
        senderConfigProps.put(ProducerConfig.ACKS_CONFIG, acks);
        senderConfigProps.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        senderConfigProps.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, JsonSerializer.class);
        return senderConfigProps;
        //return new DefaultKafkaProducerFactory<>(senderConfigProps);
    }

    @Bean
    public KafkaSender<String, Order> kafkaEventProducer() {
        SenderOptions<String, Order> senderOptions = SenderOptions.create(producerFactoryString());
        return KafkaSender.create(senderOptions);
    }

//    @Bean
//    public KafkaTemplate<String, Object> kafkaTemplateString() {
//        return new KafkaTemplate<>(producerFactoryString());
//    }
    @Bean
    public Map<String, Object> consumerFactory() {
        Map<String, Object> receiverConfigProps = new HashMap<>();
        receiverConfigProps.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
        receiverConfigProps.put(ConsumerConfig.GROUP_ID_CONFIG, groupId);
        receiverConfigProps.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
        receiverConfigProps.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
        return receiverConfigProps;
    }

    @Bean
    public KafkaReceiver<String, String> kafkaEventReceiver(@Value("${kafka.topic.input}") String posLogTopic) {
        ReceiverOptions<String, String> receiverOptions = ReceiverOptions.create(consumerFactory());
        receiverOptions.maxCommitAttempts(3);
        return KafkaReceiver.create(receiverOptions.addAssignListener(Collection::iterator).subscription(Collections.singleton(posLogTopic)));
    }

//    @Bean
//    public ConcurrentKafkaListenerContainerFactory<String, String> kafkaListenerContainerFactory() {
//        ConcurrentKafkaListenerContainerFactory<String, String> factory = new ConcurrentKafkaListenerContainerFactory<>();
//        factory.setConsumerFactory(consumerFactory());
//        return factory;
//    }
}
