package com.afp.ordermanagement.reactivekafkaservice;

import com.afp.ordermanagement.controller.OrderController;
import com.afp.ordermanagement.model.Item;
import com.afp.ordermanagement.model.Order;
import com.afp.ordermanagement.model.OrderStatus;
import com.afp.ordermanagement.repository.OrderRepository;
import com.afp.ordermanagement.service.OrderService;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigurationPackage;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.kafka.receiver.KafkaReceiver;
import reactor.kafka.receiver.ReceiverRecord;

import java.util.*;

@Service
@Slf4j
public class Receiver {

    @Autowired
    private KafkaReceiver<String, String> kafkaReceiver;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderRepository orderRepository;


    @EventListener(ApplicationStartedEvent.class)
    public void consumeOrderStatus() {
        kafkaReceiver.receive()
                .doOnNext(record -> System.out.println(record))
                .doOnNext(record -> log.info("record.value(): {} ", record.value()))
                .doOnNext(record -> updateOrderStatus(record.value()))
                .doOnError(throwable -> System.out.println(throwable.getMessage()))
                .subscribe();
    }




    private void updateOrderStatus(String orderObj) {
        try {
            //deserialize kafka message into java object using ObjectMapper
            ObjectMapper objectMapper = new ObjectMapper();
            Order order = objectMapper.readValue(orderObj, Order.class);
            log.info("ORDER objectMapper {}", order);
            String orderId = order.getId();

            Mono<Order> updated = orderService.updateOrderByOrderId(orderId, order);
            updated.block();

        } catch (Exception e) {
            log.error("Caught error", e);
        }
    }





}
