package com.afp.ordermanagement.UNIT_TESTS.helper;

import com.github.javafaker.Faker;
//import com.sun.tools.javac.util.List;
import org.junit.jupiter.api.Assertions;
import static org.junit.jupiter.api.Assertions.*;


import javax.validation.ConstraintViolation;
import javax.validation.Validation;
import javax.validation.Validator;
import java.lang.reflect.ParameterizedType;
import java.util.List;

public abstract class ValidatableEntity<E> {
    private final Class<E> childClass;

    static private final Faker FAKER = new Faker();

    @SuppressWarnings("unchecked")
    public ValidatableEntity() {
        this.childClass = (
                (Class<E>) (
                        (ParameterizedType) this.getClass()
                        .getGenericSuperclass()
                )
                        .getActualTypeArguments()[0]
        );
    }

    private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

    // assertFieldValidation(String fieldName, Object input, boolean isValidInput)
    public ConstraintViolation<E> createViolation(String fieldName, Object fieldValue) {
        return validator
                .validateValue(this.childClass, fieldName, fieldValue)
                .stream()
                .findAny()
                .orElse(null);
    }

    // assertFieldValidation(String fieldName, Object input, boolean isValidInput)
    public void assertFieldValidation(String fieldName, Object input, boolean expected) {
        boolean isValid = validator.validateValue(this.childClass, fieldName, input).isEmpty();
        String v = expected ? "valid" : "invalid";
        String errMsg = String.format("Expected '%s' to be a %s value for field '%s'", input, v, fieldName);
        if (isValid != expected) fail(errMsg);
    }
}
